# Copyright (c) 2008-2009 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: testing.py 16730 2009-07-24 11:49:49Z ctheune $

import os
import zope.app.container.interfaces
import zope.app.testing.functional
import zope.app.testing.xmlrpc
import zope.component
import zope.event
import zope.lifecycleevent
import zope.testing
from zope.testing import doctest

import gocept.directory.interfaces
import gocept.directory.resourcegroup


class AuthenticatableTestTransport(zope.app.testing.xmlrpc.ZopeTestTransport):

    user = None
    password = None

    def get_host_info(self, host):
        host, extra_headers, x509 = (
            zope.app.testing.xmlrpc.ZopeTestTransport.get_host_info(self, host))
        if self.user:
            if extra_headers is None:
                extra_headers = {}
            extra_headers['Authorization'] = 'Basic %s:%s' % (self.user,
                                                        self.password)
        return host, extra_headers, x509


def ServerProxy(uri, transport=None, encoding=None, verbose=0,
                allow_none=0, handleErrors=True, user=None, password=None):
        if transport is None:
            transport = AuthenticatableTestTransport()
        transport.user = user
        transport.password = password
        return zope.app.testing.xmlrpc.ServerProxy(
            uri, transport, encoding, verbose, allow_none, handleErrors)

DataLayer = zope.app.testing.functional.ZCMLLayer(
    os.path.join(os.path.dirname(__file__), 'ftesting.zcml'),
    __name__, 'DataLayer', allow_teardown=True)


def add_ftest(suite, filename):
    add_test(suite, filename,
             zope.app.testing.functional.FunctionalDocFileSuite,
             DataLayer)


def add_utest(suite, filename):
    add_test(suite, filename, zope.testing.doctest.DocFileSuite, None)


def add_test(suite, filename, suite_class, layer):
    """Add tests found in filename to test suite."""
    test = suite_class(
        filename,
        optionflags=(doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
                     doctest.INTERPRET_FOOTNOTES))
    if layer:
        test.layer = layer
    suite.addTest(test)


def set_site(func):
    "Decorator which does the set-site-dance."
    def decorated(*args, **kw):
        old_site = zope.app.component.hooks.getSite()
        site = zope.app.testing.functional.getRootFolder()['directory']
        try:
            zope.app.component.hooks.setSite(site)
            return func(*args, **kw)
        finally:
            zope.app.component.hooks.setSite(old_site)
    return decorated


def create_helper(container, obj):
    zope.event.notify(zope.lifecycleevent.ObjectCreatedEvent(obj))
    chooser = zope.app.container.interfaces.INameChooser(container)
    name = chooser.chooseName('', obj)
    container[name] = obj
    return container[name]



@set_site
def create_user_account(username, owner, password=u''):
    container = zope.component.getUtility(
        gocept.directory.interfaces.IUnixAccounts)
    account = gocept.directory.useraccount.UserAccount(username, password)
    account.owner = owner
    return create_helper(container, account)
