# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import unittest
import gocept.directory.app
import gocept.directory.nodes


class NodeTests(unittest.TestCase):

    def setUp(self):
        self.directory = gocept.directory.app.Directory()
        self.directory['rg'] = self.rg = gocept.directory.resourcegroup.ResourceGroup()

    def test_global_id_consistency_no_nodes(self):
        check = gocept.directory.nodes.GlobalIDConsistency(self.directory)
        self.assertEquals([], check.errors)

    def test_global_id_consistency_one_node(self):
        self.rg['node1'] = node1 = gocept.directory.nodes.Node()
        node1.global_id = 1
        node1.name = 'node1'

        check = gocept.directory.nodes.GlobalIDConsistency(self.directory)
        self.assertEquals([], check.errors)

    def test_global_id_consistency_two_nodes_no_conflict(self):
        self.rg['node1'] = node1 = gocept.directory.nodes.Node()
        node1.global_id = 1
        node1.name = 'node1'

        self.rg['node2'] = node2 = gocept.directory.nodes.Node()
        node2.global_id = 2
        node2.name = 'node2'

        check = gocept.directory.nodes.GlobalIDConsistency(self.directory)
        self.assertEquals([], check.errors)

    def test_global_id_consistency_two_nodes_conflict(self):
        self.rg['node1'] = node1 = gocept.directory.nodes.Node()
        node1.name = 'node1'
        node1.global_id = 1

        self.rg['node2'] = node2 = gocept.directory.nodes.Node()
        node2.name = 'node2'
        node2.global_id = 1

        check = gocept.directory.nodes.GlobalIDConsistency(self.directory)
        self.assertEquals(['Duplicate node ID: 1 by node1, node2'], check.errors)


def test_suite():
    return unittest.makeSuite(NodeTests)
